@@ -20,6 +20,7 @@
 
 #include <string.h>
 #include <gtk/gtkx.h>
+#include <panel-applets-manager.h>
 #include "panel-applet-container.h"
 #include "panel-marshal.h"
 
@@ -29,7 +30,10 @@ struct _PanelAppletContainerPrivate {
 	guint       name_watcher_id;
 	gchar      *bus_name;
 
+	gchar      *iid;
+	gboolean    out_of_process;
 	guint32     xid;
+	guint32     uid;
 	GtkWidget  *socket;
 
 	GHashTable *pending_ops;
@@ -52,9 +56,7 @@ typedef struct {
 
 static const AppletPropertyInfo applet_properties [] = {
 	{ "settings-path", "SettingsPath" },
-	{ "prefs-key",     "PrefsKey" },
 	{ "orient",        "Orient" },
-	{ "size",          "Size" },
 	{ "size-hints",    "SizeHints" },
 	{ "background",    "Background" },
 	{ "flags",         "Flags" },
@@ -84,19 +86,32 @@ panel_applet_container_init (PanelAppletContainer *container)
 {
 	container->priv = PANEL_APPLET_CONTAINER_GET_PRIVATE (container);
 
-	container->priv->socket = gtk_socket_new ();
-	g_signal_connect_swapped (container->priv->socket, "plug-removed",
-				  G_CALLBACK (panel_applet_container_plug_removed),
-				  container);
-
 	container->priv->pending_ops = g_hash_table_new_full (g_direct_hash,
 							      g_direct_equal,
 							      NULL,
 							      (GDestroyNotify) g_object_unref);
+}
 
-	gtk_container_add (GTK_CONTAINER (container),
-			   container->priv->socket);
-	gtk_widget_show (container->priv->socket);
+static void
+panel_applet_container_setup (PanelAppletContainer *container)
+{
+	if (container->priv->out_of_process) {
+		container->priv->socket = gtk_socket_new ();
+
+		g_signal_connect_swapped (container->priv->socket,
+		                          "plug-removed",
+		                          G_CALLBACK (panel_applet_container_plug_removed),
+		                          container);
+
+		gtk_container_add (GTK_CONTAINER (container), container->priv->socket);
+		gtk_widget_show (container->priv->socket);
+	} else {
+		GtkWidget *applet;
+
+		applet = panel_applets_manager_get_applet_widget (container->priv->iid, container->priv->uid);
+
+		gtk_container_add (GTK_CONTAINER (container), applet);
+	}
 }
 
 static void
@@ -134,6 +149,11 @@ panel_applet_container_dispose (GObject *object)
 		container->priv->bus_name = NULL;
 	}
 
+	if (container->priv->iid) {
+		g_free (container->priv->iid);
+		container->priv->iid = NULL;
+	}
+
 	if (container->priv->name_watcher_id > 0) {
 		g_bus_unwatch_name (container->priv->name_watcher_id);
 		container->priv->name_watcher_id = 0;
@@ -339,6 +359,8 @@ on_proxy_appeared (GObject      *source_object,
 	g_simple_async_result_complete (result);
 	g_object_unref (result);
 
+	panel_applet_container_setup (container);
+
 	if (container->priv->xid > 0) {
 		gtk_socket_add_id (GTK_SOCKET (container->priv->socket),
 				   container->priv->xid);
@@ -371,7 +393,12 @@ get_applet_cb (GObject      *source_object,
 	}
 
 	container = PANEL_APPLET_CONTAINER (g_async_result_get_source_object (G_ASYNC_RESULT (result)));
-	g_variant_get (retvals, "(&ou)", &applet_path, &container->priv->xid);
+	g_variant_get (retvals,
+	               "(&obuu)",
+	               &applet_path,
+	               &container->priv->out_of_process,
+	               &container->priv->xid,
+	               &container->priv->uid);
 
 	g_dbus_proxy_new (connection,
 			  G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
@@ -424,7 +451,7 @@ on_factory_appeared (GDBusConnection   *connection,
 				PANEL_APPLET_FACTORY_INTERFACE,
 				"GetApplet",
 				data->parameters,
-				G_VARIANT_TYPE ("(ou)"),
+				G_VARIANT_TYPE ("(obuu)"),
 				G_DBUS_CALL_FLAGS_NONE,
 				-1,
 				data->cancellable,
@@ -481,6 +508,7 @@ panel_applet_container_get_applet (PanelAppletContainer *container,
 
 	bus_name = g_strdup_printf (PANEL_APPLET_BUS_NAME, factory_id);
 
+	container->priv->iid = g_strdup (iid);
 	container->priv->name_watcher_id =
 		g_bus_watch_name (G_BUS_TYPE_SESSION,
 				  bus_name,
